'''for all things dicey.  this contains DiceInfo and DiceTable and add_dice.'''
from __future__ import absolute_import


from dicetables.longintmath import LongIntTable

class ProtoDie(object):
    '''a blanket object for any kind of die so that different types of Die can
    be compared.  all Die objects need these five methods. str and repr'''
    def get_size(self):
        '''return the size of the die'''
        raise NotImplementedError
    def get_weight(self):
        '''return the total weight of the die, if weighted'''
        raise NotImplementedError
    def tuple_list(self):
        '''return an ordered tuple list of [(die, weight) ... ] with zero weights
        removed'''
        raise NotImplementedError
    def weight_info(self):
        '''return detailed info of how the die is weighted'''
        raise NotImplementedError
    def multiply_str(self, number):
        '''return a string that is the die string multiplied by a number. i.e.,
        D6+1 times 3 is 3D6+3'''
        raise NotImplementedError
    def __str__(self):
        raise NotImplementedError
    def __repr__(self):
        raise NotImplementedError
    def __hash__(self):
        return hash('hash of {!r}, {}, {}, {}'.format(self, self.get_size(),
                                                      self.get_weight(),
                                                      self.tuple_list()))

    def __lt__(self, other):
        '''Dice are compared by size, then weight, then tuple_list, and finally
        repr'''
        return (
            (self.get_size(), self.get_weight(), self.tuple_list(), repr(self)) <
            (other.get_size(), other.get_weight(), other.tuple_list(), repr(other))
            )
    def __eq__(self, other):
        return (
            (self.get_size(), self.get_weight(), self.tuple_list(), repr(self)) ==
            (other.get_size(), other.get_weight(), other.tuple_list(), repr(other))
            )
    def __ne__(self, other):
        return not self == other
    def __le__(self, other):
        return self < other or self == other
    def __gt__(self, other):
        return not self <= other
    def __ge__(self, other):
        return self == other or self > other

class Die(ProtoDie):
    '''makes instance of a die.  it has a size and a weight of 0 (for comparing
    with weighted dice which have non-zero weight)'''
    def __init__(self, die_size):
        '''die size is a positive int. creates a die containing 1 to die_size
        values'''
        self._die_size = die_size
        self._weight = 0
    def get_size(self):
        '''returns the size of the die'''
        return self._die_size
    def get_weight(self):
        '''returns the weight of the die'''
        return self._weight
    def tuple_list(self):
        '''returns the tuple list that is the dice values'''
        return [(value, 1) for value in range(1, self._die_size + 1)]

    def weight_info(self):
        '''returns detailed weight info, indented'''
        return str(self) + '\n    No weights'
    def multiply_str(self, number):
        '''return the str of die times a number. 5, D6+3 --> 5D6+15'''
        return '{}{}'.format(number, self)
    def __str__(self):
        return 'D{}'.format(self._die_size)
    def __repr__(self):
        return 'Die({})'.format(self.get_size())

class ModDie(Die):
    '''a Die with a modifier.  i.e. D6-3.'''
    def __init__(self, die_size, modifier):
        '''as Die init, die_size is a positive int.  modifier is an int'''
        Die.__init__(self, die_size)
        self._mod = modifier

    def get_modifier(self):
        '''returns the modifier of a modded die'''
        return self._mod
    def tuple_list(self):
        '''returns the tuple list with the values adjusted by the modifier'''
        return [(value + self._mod, 1)
                for value in range(1, self._die_size + 1)]

    def multiply_str(self, number):
        '''return the str of die times a number. 5, D6+3 --> 5D6+15'''
        return '{}D{}{:+}'.format(number, self._die_size, number * self._mod)
    def __str__(self):
        return 'D{0}{1:+}'.format(self._die_size, self._mod)
    def __repr__(self):
        return 'ModDie({}, {})'.format(self.get_size(), self.get_modifier())

class WeightedDie(ProtoDie):
    '''stores and returns info for a weighted die.'''
    def __init__(self, dictionary_input):
        '''dictionary input is a dictionary of value:weight. values are positive
        int and weights are zero or positive ints.
        {1:3, 2:0, 3:1} creates a D3 that rolls a one 3 times more than a three
        and never rolls a two.'''
        self._dic = dictionary_input.copy()
        self._die_size = max(self._dic.keys())
        self._weight = sum(self._dic.values())

    def get_size(self):
        '''returns the size of the die'''
        return self._die_size
    def get_weight(self):
        '''returns the weight of the die'''
        return self._weight

    def tuple_list(self):
        '''returns the tuple list that is the dice values'''
        out = []
        for value, weight in self._dic.items():
            if weight != 0:
                out.append((value, weight))
        out.sort()
        return out

    def weight_info(self):
        '''returns detailed weight info, indented'''
        out = str(self) + '\n'
        for roll in range(1, self._die_size + 1):
            out += ('    a roll of {} has a weight of {}\n'
                    .format(roll, self._dic.get(roll, 0)))
        return out.rstrip('\n')
    def multiply_str(self, number):
        '''return the str of die times a number. 5, D6+3 --> 5D6+15'''
        return '{}{}'.format(number, self)
    def __str__(self):
        return 'D{}  W:{}'.format(self._die_size, self._weight)
    def __repr__(self):
        new_dic = {}
        for roll in range(1, self.get_size() + 1):
            new_dic[roll] = self._dic.get(roll, 0)
        return 'WeightedDie({})'.format(new_dic)


class ModWeightedDie(WeightedDie):
    '''stores and returns info for a weighted die. and a modifier modifies the
    values'''
    def __init__(self, dictionary_input, modifier):
        '''dictionary input is a dictionary of positive ints, value:weight.
        modifier is an int that modifies values. {1:3, 2:1, 3:1}, -3 creates a
        D3 that rolls a one(-3) 3 times more than a two(-3) or a three(-3).'''
        WeightedDie.__init__(self, dictionary_input)
        self._mod = modifier
    def get_modifier(self):
        '''returns the modifier on the die'''
        return self._mod
    def tuple_list(self):
        '''returns the tuple list that is the dice values adjust by the mod'''
        out = []
        for value, weight in self._dic.items():
            if weight != 0:
                out.append((value + self._mod, weight))
        out.sort()
        return out

    def multiply_str(self, number):
        '''return the str of die times a number. 5, D6+3 --> 5D6+15'''
        return '{0}D{1}{2:+}  W:{3}'.format(number, self._die_size,
                                            number * self._mod, self._weight)
    def __str__(self):
        return 'D{0}{1:+}  W:{2}'.format(self._die_size, self._mod, self._weight)
    def __repr__(self):
        to_fix = super(ModWeightedDie, self).__repr__()[:-1]
        return 'Mod' + to_fix + ', {})'.format(self.get_modifier())

class StrongDie(ProtoDie):
    '''stores and returns info for a Die that acts like a different Die with
    many votes.'''
    def __init__(self, input_die, multiplier):
        '''multiplier is a positive int.  input_die is any Die derived from
        ProtoDie or it's descendents. StrongDie(ModDie(3, -1), 2) would make a
        D3-1 with twice the influence of a regular die.  so it would roll
        0, 2, 4.  die size and weight are still the same.'''
        self._original = input_die
        self._multiply = multiplier
    def get_size(self):
        '''returns the size of the die'''
        return self.get_original().get_size()
    def get_weight(self):
        '''returns the weight of the die'''
        return self._original.get_weight()
    def get_multiplier(self):
        '''retruns the multiplier for the die'''
        return self._multiply
    def get_original(self):
        '''returns an instance of the original die'''
        return self._original
    def tuple_list(self):
        '''returns the tuple list that is the dice values'''
        old = self._original.tuple_list()
        return [(pair[0] * self.get_multiplier(), pair[1]) for pair in old]

    def weight_info(self):
        '''returns detailed weight info, indented'''
        return (self._original.weight_info().replace(
            str(self._original), str(self)))

    def multiply_str(self, number):
        '''return the str of die times a number. 5, D6+3 --> 5D6+15'''
        return '({})X{}'.format(self.get_original().multiply_str(number),
                                self.get_multiplier())
    def __str__(self):
        return '({})X{}'.format(self.get_original(), self.get_multiplier())
    def __repr__(self):
        return 'StrongDie({!r}, {})'.format(self.get_original(),
                                            self.get_multiplier())

class DiceTable(LongIntTable):
    '''this is a LongIntTable with a list that holds information about the dice
    added to it and removed from it.'''
    def __init__(self):
        LongIntTable.__init__(self, {0:1})
        self._dice_list = {}

    def update_list(self, add_number, new_die):
        '''adds die and number of dice to the list. if die is already in list,
        adds old and new number together.'''
        self._dice_list[new_die] = self._dice_list.get(new_die, 0) + add_number
        if self._dice_list[new_die] == 0:
            del self._dice_list[new_die]

    def get_list(self):
        '''return copy of dice list. a list of tuples, (die, number of dice)'''
        return sorted(self._dice_list.items())
    def number_of_dice(self, query_die):
        '''returns the number of that die in the dice list, or zero if not in
        the list'''
        return self._dice_list.get(query_die, 0)

    def weights_info(self):
        '''return detailed info of dice in the list'''
        out_str = ''
        for die, number in self.get_list():
            adjusted = die.weight_info().replace(str(die),
                                                 die.multiply_str(number))
            out_str += adjusted + '\n\n'
        return out_str.rstrip('\n')


    def __str__(self):
        out_str = ''
        for die, number in self.get_list():
            out_str += '{}\n'.format(die.multiply_str(number))
        return out_str.rstrip('\n')

    def add_die(self, num, die):
        '''adds the die to the table num times and updates the table, list.  die
        is a  Die or WeightedDie and num is a positive int or 0.'''
        self.add(num, die.tuple_list())
        self.update_list(num, die)

    def remove_die(self, num, die):
        '''die is a  Die or WeightedDie and num is a positive int or 0.first
        makes sure the dice to be removed are in the list. then removes those
        dice from the list and the table.'''
        if self.number_of_dice(die) - num < 0:
            raise ValueError('dice not in table, or removed too many dice')
        else:
            self.remove(num, die.tuple_list())
            self.update_list(-num, die)








# pylint: disable=missing-docstring, invalid-name, too-many-public-methods
'''tests for the tableinfo.py module'''

from __future__ import absolute_import


import unittest
from dicetables import LongIntTable
import dicetables.tableinfo as ti

class TestTableInfo(unittest.TestCase):
    def test_scinote_returns_exp_for_val_le_ten_to_neg_four(self):
        self.assertEqual(ti.scinote(10**-4), '1.000e-04')
    def test_scinote_returns_fixed_pt_for_val_gt_ten_to_neg_four(self):
        self.assertEqual(ti.scinote(10**-3), '0.001000')
    def test_scinote_edge_case_at_ten_to_neg_four(self):
        self.assertEqual(ti.scinote(9.99952*10**-4), '0.001000')
        self.assertEqual(ti.scinote(9.99949*10**-4), '9.999e-04')
    def test_scinote_returns_zero(self):
        self.assertEqual(ti.scinote(2*10**-500), '0.0')
    def test_scinote_returns_commaed_int_at_less_than_ten_million(self):
        self.assertEqual(ti.scinote(2300001), '2,300,001')
    def test_scinote_return_commaed_int_for_float_longer_than_dig_len(self):
        self.assertEqual(ti.scinote(1.234567e+6, 4), '1,234,567')
    def test_scinote_rounds_commaed_int_for_float_larger_than_dig_len(self):
        self.assertEqual(ti.scinote(1.2345675e+6, 4), '1,234,568')
    def test_scinote_as_above_with_negative_number(self):
        self.assertEqual(ti.scinote(-1.2345675e+6, 4), '-1,234,568')
    def test_scinote_rounds_commaed_float_for_dig_len_larger_than_tens_place(self):
        self.assertEqual(ti.scinote(123456.789, 8), '123,456.79')
    def test_scinote_rounds_commaed_float_and_retains_zeros(self):
        self.assertEqual(ti.scinote(123456.000, 8), '123,456.00')
    def test_scinote_commaed_float_adds_missing_zeros(self):
        self.assertEqual(ti.scinote(123456., 8), '123,456.00')
    def test_scinote_edge_case_non_exp_float_rounds_up_and_adds_a_digit(self):
        self.assertEqual(ti.scinote(99.999), '100.0')
    def test_scinote_doesnt_add_digits_when_dig_len_larger_than_int(self):
        self.assertEqual(ti.scinote(1234, 6), '1,234')
    def test_scinote_edge_case_at_10_to_7th_1(self):
        self.assertEqual(ti.scinote(9.999999999e+6, 10), '9,999,999.999')
    def test_scinote_edge_case_at_10_to_7th_2(self):
        self.assertEqual(ti.scinote(9.999999999e+6, 3), '1.00e+07')
    def test_scinote_edge_case_at_10_to_7th_3(self):
        self.assertEqual(ti.scinote(10000000), '1.000e+07')
    def test_scinote_edge_case_at_10_to_7th_negative_number_1(self):
        self.assertEqual(ti.scinote(-9.999999999e+6, 10), '-9,999,999.999')
    def test_scinote_edge_case_at_10_to_7th_negative_number_2(self):
        self.assertEqual(ti.scinote(-9.999999999e+6, 3), '-1.00e+07')
    def test_scinote_edge_case_at_10_to_7th_negative_number_3(self):
        self.assertEqual(ti.scinote(-10000000), '-1.000e+07')
    def test_scinote_returns_exponent_form_for_number_over_10_to_7th(self):
        self.assertEqual(ti.scinote(1.2345678e+7, 6), '1.23457e+07')
    def test_scinote_fills_zeros_to_dig_len_over_10_to_7th(self):
        self.assertEqual(ti.scinote(1.2e+7, 6), '1.20000e+07')
    def test_scinote_large_number(self):
        self.assertEqual(ti.scinote(5*10**1000, 4), '5.000e+1000')
    def test_scinote_large_number_negative(self):
        self.assertEqual(ti.scinote(-56*10**1000, 4), '-5.600e+1001')
    def test_scinote_large_number_rounds_like_it_should(self):
        self.assertEqual(ti.scinote(555551*10**1000, 4), '5.556e+1005')
    def test_scinote_annoying_edge_case_due_to_python_rounding_errors(self):
        #the issue is python rounds the binary approximation of the float
        #so it's not rounding 5.5550000000000 it's rounding a slightly
        #smaller binary number.
        self.assertEqual(ti.scinote(5555*10**1000, 3), '5.55e+1003')
        self.assertEqual(ti.scinote(-5555*10**1000, 3), '-5.55e+1003')

    def test_list_to_string_returns_single_number(self):
        self.assertEqual(ti.list_to_string([1, 1, 1]), '1')
    def test_list_to_string_puts_parentheses_around_negative_numbers(self):
        self.assertEqual(ti.list_to_string([-1]), '(-1)')
    def test_list_to_string_returns_proper_output_for_a_run_of_numbers(self):
        self.assertEqual(ti.list_to_string([1, 1, 2, 3, 4, 5]), '1-5')
    def test_list_to_string_returns_values_separated_by_commas(self):
        the_list = [-5, -4, -3, -1, 0, 1, 2, 3, 5]
        self.assertEqual(ti.list_to_string(the_list), '(-5)-(-3), (-1)-3, 5')
    def test_list_to_string_returns_numbers_with_commas(self):
        the_list = [-1234567, 1234567]
        self.assertEqual(ti.list_to_string(the_list), '(-1,234,567), 1,234,567')

    def test_full_table_string_returns_empty_str_for_empty_table(self):
        self.assertEqual(ti.full_table_string(LongIntTable({})), '')
    def test_full_table_string_right_justifies_all_values_also_no_zeroes(self):
        table = LongIntTable({1:10, 10:200, 1000:3000})
        self.assertEqual(ti.full_table_string(table, zeroes=False),
                         '   1: 10\n  10: 200\n1000: 3,000\n')
    def test_full_table_string_includes_zeroes_as_requested(self):
        table = LongIntTable({1:1, 3:1})
        self.assertEqual(ti.full_table_string(table, zeroes=True),
                         '1: 1\n2: 0.0\n3: 1\n')
    def test_full_table_string_edge_case(self):
        table = LongIntTable({0:1})
        self.assertEqual(ti.full_table_string(table), '0: 1\n')

    def test_graph_pts_raises_error_for_empty_table(self):
        with self.assertRaises(ValueError) as cm:
            ti.graph_pts(LongIntTable({}))
        self.assertEqual(cm.exception.args[0], 'empty table')
    def test_graph_pts_adds_zeroes(self):
        table = LongIntTable({1: 1, 3: 1})
        self.assertEqual(ti.graph_pts(table, percent=False),
                         [(1, 2, 3), (1, 0, 1)])
    def test_graph_pts_doesnot_add_zeroes_on_request(self):
        table = LongIntTable({1: 1, 3: 1})
        self.assertEqual(ti.graph_pts(table, percent=False, zeroes=False),
                         [(1, 3), (1, 1)])
    def test_graph_pts_return_percent(self):
        table = LongIntTable({1: 1, 3: 3})
        self.assertEqual(ti.graph_pts(table, percent=True),
                         [(1, 2, 3), (25, 0, 75)])
    def test_graph_pts_returns_percent_with_large_int_values(self):
        table = LongIntTable({1: 10**1000, 3: 3 * 10**1000})
        self.assertEqual(ti.graph_pts(table), [(1, 2, 3), (25, 0, 75)])
    def test_graph_pts_returns_xy_axes_on_request(self):
        table = LongIntTable({1:1, 3:3})
        self.assertEqual(ti.graph_pts(table, axes=True), [(1, 2, 3), (25, 0, 75)])
    def test_graph_pts_returns_xypts_on_request(self):
        table = LongIntTable({1: 1, 3: 3})
        self.assertEqual(ti.graph_pts(table, axes=False),
                         [(1, 25), (2, 0), (3, 75)])
    def test_graph_pts_not_exact_x_vals_equals_exact_x_vals(self):
        table = LongIntTable({1:2, 3:4, 5:6})
        self.assertEqual(ti.graph_pts(table, exact=True)[0],
                         ti.graph_pts(table, exact=False)[0])
    def test_graph_pts_not_exact_equals_exact_to_ten_dec_places(self):
        table = LongIntTable(dict([(num, num**2) for num in range(1000)]))
        exact_y = ti.graph_pts(table, exact=True)[1]
        inexact_y = ti.graph_pts(table, exact=False)[1]
        for index in range(len(exact_y)):
            self.assertAlmostEqual(exact_y[index], inexact_y[index], places=10)
    def test_graph_pts_overflow_for_small_numbers(self):
        table = LongIntTable({1: 1, 3: 1})
        self.assertEqual(ti.graph_pts_overflow(table), ([(1, 2, 3), (1, 0, 1)], '1'))
    def test_graph_pts_overflow_for_larg_numbers(self):
        table = LongIntTable({1: 10**200, 2: 1})
        self.assertEqual(ti.graph_pts_overflow(table),
                         ([(1, 2), (10**200, 1)], '1'))
    def test_graph_pts_overflow_for_very_large_numbers(self):
        table = LongIntTable({1:10**2000, 2:1})
        self.assertEqual(ti.graph_pts_overflow(table),
                         ([(1, 2), (10**4, 0)], '1.0e+1996'))
    def test_graph_pts_overflow_raises_error_for_empty_table(self):
        with self.assertRaises(ValueError) as cm:
            ti.graph_pts_overflow(LongIntTable({}))
        self.assertEqual(cm.exception.args[0], 'empty table')

    def test_ascii_graph_helper_for_empty_table(self):
        empty_result = ti.ascii_graph_helper(LongIntTable({}))
        self.assertEqual(empty_result, [(None, 'each x represents 1 occurence')])
    def test_ascii_graph_helper_for_table_under_80(self):
        result = ti.ascii_graph_helper(LongIntTable({1: 5, 2: 80}))
        expected = [(1, '1:{}'.format(5 * 'x')), (2, '2:{}'.format(80 * 'x')),
                    (None, 'each x represents 1 occurence')]
        self.assertEqual(result, expected)
    def test_ascii_graph_helper_for_table_over_80(self):
        result = ti.ascii_graph_helper(LongIntTable({1: 5, 2: 8000}))
        expected = [(1, '1:'), (2, '2:{}'.format(80 * 'x')),
                    (None, 'each x represents 100.0 occurences')]
        self.assertEqual(result, expected)
    def tets_ascii_graph_justifies_right_for_values(self):
        result = ti.ascii_graph_helper(LongIntTable({1: 1, 2000: 1}))
        expected = [(1, '   1:x'), (2, '2000:x'),
                    (None, 'each x represents 1 occurence')]
        self.assertEqual(result, expected)
    def test_ascii_graph_works(self):
        result = ti.ascii_graph(LongIntTable({1: 1, 2: 1}))
        expected = '1:x\n2:x\neach x represents 1 occurence'
        self.assertEqual(result, expected)
    def test_ascii_graph_truncated_when_no_truncating(self):
        result = ti.ascii_graph_truncated(LongIntTable({1: 1, 2: 1}))
        expected = '1:x\n2:x\neach x represents 1 occurence'
        self.assertEqual(result, expected)
    def test_ascii_graph_truncated_when_truncating_one_side(self):
        result = ti.ascii_graph_truncated(LongIntTable({1: 1, 2: 1, 3: 8000}))
        expected = ('3:{}\neach x represents 100.0 occurences\nnot included: 1-2'
                    .format(80 * 'x'))
        self.assertEqual(result, expected)
    def test_ascii_graph_truncated_when_truncating_two_sides(self):
        result = ti.ascii_graph_truncated(LongIntTable({1: 1, 2: 1, 3: 8000, 4: 1}))
        expected = ('3:{}\n'.format(80 * 'x') +
                    'each x represents 100.0 occurences\n' +
                    'not included: 1-2 and 4')
        self.assertEqual(result, expected)
    def test_stats_empty_table(self):
        result = ti.stats(LongIntTable({}), [1, 2, 3])
        expected = ('1-3', '0.0', '0.0', 'infinity', '0.0')
        self.assertEqual(result, expected)
    def test_stats_not_in_table(self):
        result = ti.stats(LongIntTable({1: 1}), [2])
        expected = ('2', '0.0', '1', 'infinity', '0.0')
        self.assertEqual(result, expected)
    def test_stats_does_not_repeat_values(self):
        result = ti.stats(LongIntTable({1: 1}), [1, 1])
        expected = ('1', '1', '1', '1.000', '100.0')
        self.assertEqual(result, expected)
    def test_stats_includes_values_not_in_table(self):
        result = ti.stats(LongIntTable({1: 1, 2: 1}), [0, 1])
        expected = ('0-1', '1', '2', '2.000', '50.00')
        self.assertEqual(result, expected)
    def test_stats_works_for_large_values(self):
        result = ti.stats(LongIntTable({1: 10**1000, 2: (10**1002-10**1000)}), [1])
        expected = ('1', '1.000e+1000', '1.000e+1002', '100.0', '1.000')
        self.assertEqual(result, expected)

if __name__ == '__main__':
    unittest.main()




''' this module contains the class LongIntTable and longint math that the table
needs to deal with it's BFN'''
#these functions are concerned with float-math for long ints.
from decimal import Decimal as dec
def _convert_back(num):
    '''helper function.  takes a Decimal and returns float if
    possible, else, long_int'''
    if float(num) == float('inf') or float(num) == float('-inf'):
        return int(num)
    else:
        return float(num)

def long_int_div(numerator, denominator):
    '''returns a float division of numbers even if they are over 1e+308'''
    ans = dec(numerator) / dec(denominator)
    return _convert_back(ans)

def long_int_times(number1, number2):
    '''returns a float times of numbers even if they are over 1e+308'''
    ans = dec(number1) * dec(number2)
    return _convert_back(ans)

def long_int_pow(number, exponent):
    '''returns a float exponent of numbers even if they are over 1e+308'''
    ans = dec(number)**dec(exponent)
    return _convert_back(ans)



class LongIntTable(object):
    '''a table of big fucking numbers and some math function for them.
    The table implicitly contains 0 occurences of all unassigned intergers.
    THIS TABLE SHOULD ONLY CONTAIN INT OR LONG.  it will not raise errors if you
    put in other values, but there is not telling what problems will happen.'''

    def __init__(self, seed_dictionary):
        '''seed_dictionary is a dictionary of ints. frequencies MUST BE POSITIVE.
        {value1: (frequency of value1), value2: (frequency of value 2), ...}'''
        self._table = seed_dictionary.copy()

    def values(self):
        '''return the all the values, in order, that have non-zero frequency'''
        the_values = []
        for value, freq in self._table.items():
            if freq != 0:
                the_values.append(value)
        the_values.sort()
        return the_values
    def values_min(self):
        '''returns the min value'''
        if self.values() == []:
            return None
        else:
            return self.values()[0]
    def values_max(self):
        '''returns the max value'''
        if self.values() == []:
            return None
        else:
            return self.values()[-1]
    def values_range(self):
        '''returns a tuple of min and max values'''
        return self.values_min(), self.values_max()

    def frequency(self, value):
        '''Returns a tuple of the value and it's frequency.'''
        return value, self._table.get(value, 0)
    def frequency_range(self, start, stopbefore):
        '''Returns a list of tuples (value,frequency).
        Like regular range function, it stops before endvalue.'''
        tuple_list = []
        for value in range(start, stopbefore):
            tuple_list.append(self.frequency(value))
        return tuple_list
    def frequency_all(self):
        '''Returns a list of tuples IN ORDER for all non-zero-frequency
        values in table.'''
        value_list = self.values()
        tuple_list = []
        for value in value_list:
            tuple_list.append(self.frequency(value))
        return tuple_list
    def frequency_highest(self):
        '''Returns a tuple of (one of) the value(s) with the highest frequency,
        and it's frequency'''
        hf_val, hf_freq = None, 0
        for value, frequency in self._table.items():
            if frequency > hf_freq:
                hf_val, hf_freq = value, frequency
        return hf_val, hf_freq

    def total_frequency(self):
        '''returns the sum all the freuencies in a table'''
        all_freq = self._table.values()
        return sum(all_freq)

    def __str__(self):
        return ('table from %s to %s' %
                (self.values_min(), self.values_max()))

    def mean(self):
        '''i mean, don't you just sometimes look at a table of values
        and wonder what the mean is?'''
        numerator = sum([value*freq for value, freq in self._table.items()])
        denominator = self.total_frequency()
        if denominator == 0:
            raise ZeroDivisionError('there are no values in the table')
        return long_int_div(numerator, denominator)
    def stddev(self, decimal_place=4):
        '''returns the standdard deviation of the table, with special measures
        to deal with long ints.'''
        avg = self.mean()
        extra_digits = 5
        power = len(str(self.frequency_highest()[1])) - 1
        if power < 2 * (decimal_place + extra_digits):
            factor = 1
        else:
            factor = 10**(power - (decimal_place + extra_digits))
        sqs = 0
        count = 0
        for value, frequency in self._table.items():
            sqs += (frequency//factor) * (avg - value)**2
            count += frequency
        new_count = count//factor
        return round((sqs/new_count)**0.5, decimal_place)

    def add(self, times, values):
        '''times is positive int or 0. values is a list of tuples(value, frequency)
        value and frequency are ints or longs, NO NEGATIVE FREQUENCIES ALLOWED!
        this function adds your table's values and frequency and the values's.

        here's how it works - original list event A is 3 out of 5.
        event B is 2 out of 5 or {A:3, B:5}. add {A:2, B:1} ( [A,A,B] ) this way.
        A+A = 3*2, A+B = (3*1+5*2) B+B = 5*1.  new dict = {AA:6, AB:8, BB:5}'''
        if times < 0:
            raise ValueError('times must be a positive int')
        to_add = self._check_cull_sort(values)
        def _fastest(tuple_list):
            '''returns fastest method'''
            experimentally_determined_ratio = 1.4
            number_of_values = len(tuple_list)
            total_freq = 0
            use_tuples = True
            for pair in tuple_list:
                total_freq += pair[1]
            list_ratio = long_int_div(total_freq, number_of_values)
            if list_ratio > experimentally_determined_ratio:
                return tuple_list, use_tuples
            else:
                use_tuples = False
                new_list = []
                for val, freq in tuple_list:
                    new_list = new_list + [val] * freq
                return new_list, use_tuples

        the_list, use_tuples = _fastest(to_add)
        #if a list of ints is faster, will do that
        if use_tuples:
            for _ in range(times):
                self._add_tuple_list(the_list)
        #otherwise adds by tuple
        else:
            for _ in range(times):
                self._add_a_list(the_list)
    @staticmethod
    def _check_cull_sort(tuple_list):
        '''prepares a list for add and remove.  removes zero values, raises
        errors where appropriate and returns a sorted list.'''
        new_list = []
        for val, freq in tuple_list:
            if freq < 0:
                raise ValueError('frequencies may not be negative')
            if freq != 0:
                new_list.append((val, freq))
        if new_list == []:
            raise ValueError('cannot add an empty list')
        new_list.sort()
        return new_list

    def _add_a_list(self, lst):
        '''lst is ints. takes the table.  for each int in the list, makes new
        tables with each value changed to value+int. merges those new tables
        and updates the existing table to become the merge.'''
        #so for [1,2,3], this would take {0:1,1:2} and
        #update to {1:1, 2:2} + {2:1, 3:2} + {3:1, 4:2}
        newdic = {}
        for value, current_frequency in self._table.items():
            for val in lst:
                newdic[value+val] = (newdic.get(value+val, 0)+current_frequency)
        self._table = newdic

    def _add_tuple_list(self, lst):
        '''as add_a_list, but now pass a list of tuples of ints.
        [(2,3), (5,7)] means add 2 three times and add 5 seven times. much more
        efficient if numbers repeat a lot in your list.'''
        newdic = {}
        for value, current_frequency in self._table.items():
            for val, freq in lst:
                newdic[value+val] = (newdic.get(value+val, 0)+
                                     freq*current_frequency)
        self._table = newdic

    def remove(self, times, to_remove):
        '''times is positive int or 0. values is a list of tuple(value, frequency)
        value and frequency are long or int. NO NEGATIVE FREQUENCIES ALLOWED!
        this function reverses previous adds.  if you remove something you never
        added, or remove it more times than you added it, THERE IS NO RECORD OF
        WHAT YOU ADDED AND NO ERROR WILL BE RAISED. PLEASE BE CAREFUL.'''
        if times < 0:
            raise ValueError('times must be a positive int')
        remove_now = self._check_cull_sort(to_remove)
        for _ in range(times):
            self._remove_tuple_list(remove_now)

    def _remove_tuple_list(self, tuple_list):
        '''tuple_list is a sorted list of tuples (value, frequency) with NO ZERO
        frequencies.  does the opposite of _add_tuple_list'''
        tuples_min = tuple_list[0][0]
        tuples_max = tuple_list[-1][0]

        start = self.values_min()-tuples_min
        stop = self.values_max()- tuples_max
        new_dic = {}
        for value in range(start, stop+1):
            try:
                new_dic_val = self._table[value+tuples_min]
                for tup_val, tup_weight in tuple_list[1:]:
                    the_diff = tup_val - tuples_min
                    new_dic_val = new_dic_val - (new_dic.get(value - the_diff, 0) *
                                                 tup_weight)
                new_dic[value] = new_dic_val // tuple_list[0][1]
            except KeyError:
                continue
        self._table = new_dic

    def merge(self, other):
        '''other is list of int tuples [(value, freq)].  adds all those value,
        freq to self'''
        for val, freq in other:
            self._table[val] = self._table.get(val, 0) + freq

    def update_frequency(self, value, new_freq):
        '''looks up a value, and changes its frequency to the new one'''
        self._table[value] = new_freq

    def update_value_ow(self, old_val, new_val):
        '''takes the frequency at old_val and moves it to new_val'''
        freq = self._table[old_val]
        self._table[old_val] = 0
        self._table[new_val] = freq

    def update_value_add(self, old_val, new_val):
        '''takes the frequency at old_vall and moves it to new_val where it adds
        to the frequency already at new_val'''
        freq = self._table[old_val]
        self._table[old_val] = 0
        self._table[new_val] = self._table.get(new_val, 0)+freq



 


'''functions for getting useful info from tables, ie. plot pts or stats'''
from __future__ import absolute_import

from dicetables.longintmath import long_int_div as li_div

#tools for string formatting
def scinote(num, dig_len=4):
    '''num is int, float or long.  dig_len is int and < 18 and >= 2.
    returns a string of num in a nicely readable form.  rounds to dig_len.
    note - python has a rounding bug (see tableinfo_test.py), but this
    is a DISPLAY function. also note- dig_len over 18 works but has errors in
    output unless the number is higher than 10**309.'''
    sci_power_cutoff = 7
    if num == 0:
        outpt = '0.0'
    #abs(numbers) less than one use the fixed point or exp to dig_len precision
    elif 0 < abs(num) < 1:
        exp = int('{0:.{1}e}'.format(num, dig_len - 1).split('e')[1])
        use_decimal = -3
        if exp < use_decimal:
            outpt = '{0:.{1}e}'.format(num, dig_len - 1)
        else:
            outpt = '{0:.{1}f}'.format(num, dig_len - 1 - exp)
    #1 <= abs(numbers) < sci_power_cutoff are appropriately rounded and comma-ed
    elif 1 <= abs(num) < 10**sci_power_cutoff:
        left = str(abs(num)).split('.')[0]
        int_digits = len(left)
        if dig_len > int_digits and isinstance(num, float):
            tick_over_tst = str(round(num, dig_len - int_digits))
            if tick_over_tst[0] == '1' and str(num)[0] == '9':
                int_digits += 1
            #num = float(round(num, dig_len - int_digits))
            outpt = '{0:,.{1}f}'.format(num, dig_len - int_digits)
        else:
            num = int(round(num, 0))
            #edge case workaround-else scinote(9.99e+6, 2) returns '10,000,000'
            if abs(num) == 10**sci_power_cutoff:
                outpt = '{0:.{1}e}'.format(num, dig_len - 1)
            else:
                outpt = '{:,}'.format(num)
    else:
        try:
            outpt = '{0:.{1}e}'.format(num, dig_len - 1)
        except OverflowError:
            outpt = _long_note(num, dig_len)
    return outpt

def _long_note(num, dig_len):
    '''converts long ints over +/-1e+308 to sci notation. helper to scinote'''
    num_str = str(abs(num))
    power = len(num_str) - 1
    digits = num_str[0] + '.' + num_str[1:dig_len + 10]
    digits_float = float(digits)
    if num < 0:
        digits_float *= -1
    return '{0:.{1}f}e+{2}'.format(digits_float, dig_len - 1, power)

def list_to_string(lst):
    '''outputs a list of intergers as a nice string.
    [1,2,3,7,9,10] becomes "1-3, 7, 9-10"
    [1,1,2,2,3] becomes "1-3"'''
    lst.sort()
    start_at = current = lst[0]
    tuple_list = []

    for next_one in lst[1:]:
        if current + 1 < next_one:
            tuple_list.append((start_at, current))
            start_at = next_one
        current = next_one
    tuple_list.append((start_at, current))
    def paren_negs(num):
        '''returns str(num) with parentethes around negative numbers'''
        return '({:,})'.format(num) if num < 0 else '{:,}'.format(num)
    out_list = []
    for pair in tuple_list:
        if pair[0] == pair[1]:
            out_list.append(paren_negs(pair[0]))
        else:
            out_list.append('{}-{}'.format(paren_negs(pair[0]),
                                           paren_negs(pair[1])))
    return ', '.join(out_list)



def full_table_string(table, zeroes=True):
    '''returns a string of the entire table with nicely formatted numbers'''
    min_val, max_val = table.values_range()
    if zeroes and table.values():
        the_pts = table.frequency_range(min_val, max_val + 1)
    else:
        the_pts = table.frequency_all()
    outstr = ''
    max_len = len(str(table.values_max()))
    for value, frequency in the_pts:
        outstr += '{0:>{1}}: {2}\n'.format(value, max_len, scinote(frequency))
    return outstr
def graph_pts(table, percent=True, axes=True, zeroes=True, exact=False):
    '''returns graph pts for a table.
    axes=True returns (x_axis, y_axis). axes=False returns [(x,y), (x,y)...]
    zeroes includes zero freq values from the min_val to max_val of the table
    percent=True converts y values into percent of total y values.
    exact=True/False only works with pct.  exact=False only good to ten decimal
    places, but much much faster.
    percent=False leaves as raw long ints, which will often be too large for
    graphing functions to handle.  for that case, graph_pts_overflow(table) is
    reccommended.'''
    if not table.values():
        raise ValueError('empty table')

    if zeroes:
        the_pts = table.frequency_range(table.values_min(),
                                        table.values_max() + 1)
    else:
        the_pts = table.frequency_all()
    if percent:
        temp = []
        total_freq = table.total_frequency()
        if exact:
            for value, freq in the_pts:
                temp.append((value, li_div(100 * freq, total_freq)))
        else:
            factor = 10**50
            for value, freq in the_pts:
                y_val = (freq * factor) // total_freq
                temp.append((value, (y_val*100.)/factor))
        the_pts = temp[:]
    if axes:
        return [tuple([pair[0] for pair in the_pts]),
                tuple([pair[1] for pair in the_pts])]
    else:
        return the_pts
def graph_pts_overflow(table, axes=True, zeroes=True):
    '''return graph points and the factor they are modified by to control for
    overflow problems by long ints.'''
    raw_pts = graph_pts(table, percent=False, axes=axes, zeroes=zeroes)
    factor = 1
    overflow_point = 10**300

    if table.frequency_highest()[1] > overflow_point:
        power = len(str(table.frequency_highest()[1])) - 5
        factor = 10**power
    factor_string = scinote(factor, 2)
    if axes:
        x_axis, old_y_axis = raw_pts
        new_y_axis = tuple([old_val // factor for old_val in old_y_axis])
        return [x_axis, new_y_axis], factor_string
    else:
        new_pts = [(x_val, y_val // factor) for x_val, y_val in raw_pts]
        return new_pts, factor_string

def ascii_graph_helper(table):
    '''table is a LongIntTable. makes a list of tuples which
    [(value, x's representing value), ...]'''
    output_list = []
    max_frequency = table.frequency_highest()[1]
    max_graph_height = 80
    divisor = 1
    add_s = ''
    if max_frequency > max_graph_height:
        divisor = li_div(max_frequency, max_graph_height)
        add_s = 's'
    val_len = len(str(table.values_max()))
    for value, frequency in table.frequency_all():
        num_of_xs = int(round(li_div(frequency, divisor)))
        output_list.append((value,
                            '{0:>{1}}:{2}'.format(value, val_len, num_of_xs*'x')
                           ))
    output_list.append((None,
                        'each x represents {} occurence{}'.format(
                            scinote(divisor), add_s
                            )
                       ))
    return output_list


def ascii_graph(table):
    '''table is a LongIntTable. returns a graph of x's.'''
    #for output in graph_list(table):
    #    print output[1]
    temp = [pair[1] for pair in  ascii_graph_helper(table)]
    return '\n'.join(temp)


def ascii_graph_truncated(table):
    '''table is a LongIntTable. prints a graph of x's,
    but doesn't print zero-x rolls'''
    excluded = []
    output = []
    for value, string in ascii_graph_helper(table):
        if 'x' in string:
            output.append(string)
        else:
            excluded.append(value)
    if excluded:
        output.append('not included: {}'.format(
            list_to_string(excluded).replace(',', ' and')
            ))
    return '\n'.join(output)



def stats(table, values):
    '''table is a LongIntTable. values is a list. returns tuple of strings
    (string of input values, combinations from input values, total combos,
    1 in how many chance, percent chance)'''
    total_freq = table.total_frequency()
    lst_freq = 0
    no_copies = set(values)
    for value in no_copies:
        lst_freq += table.frequency(value)[1]

    if lst_freq == 0:
        chance = 'infinity'
        pct = scinote(0)
    else:
        chance = scinote(li_div(total_freq, lst_freq))
        pct = scinote(100 * li_div(lst_freq, total_freq))
    return (list_to_string(values),
            scinote(lst_freq),
            scinote(total_freq),
            chance,
            pct)




'''All major classes for dicetables can be improted directly as dicetables.Class
or from original module as dicetables.module.Class.
DiceTable() is a LongIntTable with a list of dice added by method add_die()
  to see all methods, help(DiceTable) of note,
        stddev, mean, total_frequency
4 dice, Die, ModDie, WeightedDie, ModWeightedDie.
functions for getting info from tables:
    full_table_string, graph_pts, graph_pts_overflow, stats, ascii_graph
'''
from __future__ import absolute_import


from dicetables import dicestats
from dicetables import longintmath
from dicetables import tableinfo
from dicetables.longintmath import long_int_div, long_int_pow, long_int_times,\
                                        LongIntTable
from dicetables.dicestats import Die, ModDie, WeightedDie, ModWeightedDie,\
                                    StrongDie, ProtoDie, DiceTable
from dicetables.tableinfo import full_table_string, graph_pts, \
                                    graph_pts_overflow, stats, scinote



# pylint: disable=missing-docstring, invalid-name, too-many-public-methods
'''unittests for dicestats.py'''
from __future__ import absolute_import


import unittest
#import dicetables.dicestats as ds
import dicetables as dt

class TestDiceStats(unittest.TestCase):
    def test_die_equal(self):
        self.assertEqual(dt.Die(3) == dt.Die(3), True)
        self.assertEqual(dt.Die(0) == dt.Die(5), False)
    def test_die_not_equal(self):
        self.assertEqual(dt.Die(3) != dt.Die(10), True)
        self.assertEqual(dt.Die(5) != dt.Die(5), False)
    def test_die_lt(self):
        self.assertEqual(dt.Die(4) < dt.Die(5), True)
        self.assertEqual(dt.Die(4) < dt.Die(4), False)
        self.assertEqual(dt.Die(4) < dt.Die(3), False)
    def test_die_le(self):
        self.assertEqual(dt.Die(4) <= dt.Die(5), True)
        self.assertEqual(dt.Die(4) <= dt.Die(4), True)
        self.assertEqual(dt.Die(4) <= dt.Die(3), False)
    def test_die_gt(self):
        self.assertEqual(dt.Die(7) > dt.Die(7), False)
        self.assertEqual(dt.Die(7) > dt.Die(0), True)
        self.assertEqual(dt.Die(8) > dt.Die(7), True)
    def test_die_ge(self):
        self.assertEqual(dt.Die(4) >= dt.Die(5), False)
        self.assertEqual(dt.Die(4) >= dt.Die(4), True)
        self.assertEqual(dt.Die(4) >= dt.Die(3), True)
    def test_die_moddie_eq(self):
        self.assertEqual(dt.ModDie(4, 1) == dt.Die(4), False)
        self.assertEqual(dt.Die(4) == dt.ModDie(4, -1), False)
    def test_die_moddie_lt(self):
        self.assertEqual(dt.Die(4) < dt.ModDie(4, 1), True)
        self.assertEqual(dt.Die(4) < dt.ModDie(4, -1), False)
    def test_moddie_with_zero_mod_not_equal_die_due_to_repr(self):
        self.assertEqual(dt.ModDie(10, 0) != dt.Die(10), True)

    def test_weighteddie_eq(self):
        self.assertEqual(dt.WeightedDie({1:1}) == dt.WeightedDie({1:1}), True)
        self.assertEqual(dt.WeightedDie({1:1}) == dt.WeightedDie({2:2}), False)
    def test_weighteddie_not_equal_die_with_same_dictionary(self):
        self.assertEqual(dt.WeightedDie({1:1, 2:1}) == dt.Die(2), False)
    def test_weighteddie_lt_by_size(self):
        self.assertEqual(dt.WeightedDie({2:2}) < dt.WeightedDie({3:2}), True)
    def test_weighteddie_lt_by_weight(self):
        self.assertEqual(dt.WeightedDie({2:2}) < dt.WeightedDie({2:3}), True)
    def test_weighteddie_lt_by_tuple_list(self):
        self.assertEqual(dt.WeightedDie({1:1, 2:2}) < dt.WeightedDie({1:2, 2:1}), True)
    def test_die_lt_by_repr(self):
        self.assertEqual(dt.WeightedDie({1:1}) > dt.ModWeightedDie({1:1}, 0), True)

    def test_modweighteddie_with_zero_mod_not_equal_weighteddie(self):
        self.assertEqual(dt.WeightedDie({1:2}) == dt.ModWeightedDie({1:2}, 0), False)


    def test_die_get_size(self):
        self.assertEqual(dt.Die(6).get_size(), 6)
    def test_die_get_weight(self):
        self.assertEqual(dt.Die(10).get_weight(), 0)
    def test_die_tuple_list(self):
        self.assertEqual(dt.Die(3).tuple_list(), [(1, 1), (2, 1), (3, 1)])
    def test_die_weight_info(self):
        self.assertEqual(dt.Die(10000).weight_info(), 'D10000\n    No weights')
    def test_die_str(self):
        self.assertEqual(str(dt.Die(1234)), 'D1234')
    def test_die_repr(self):
        self.assertEqual(repr(dt.Die(123)), 'Die(123)')
    def test_die_multiply_str(self):
        self.assertEqual(dt.Die(5).multiply_str(101), '101D5')

    def test_moddie_get_modifier(self):
        self.assertEqual(dt.ModDie(7, 33).get_modifier(), 33)
    def test_moddie_tuple_list(self):
        self.assertEqual(dt.ModDie(2, 5).tuple_list(), [(1+5, 1), (2+5, 1)])
    def test_moddie_string(self):
        self.assertEqual(str(dt.ModDie(10, 0)), 'D10+0')
    def test_moddie_repr(self):
        self.assertEqual(repr(dt.ModDie(5, -1)), 'ModDie(5, -1)')
    def test_moddie_multiply_str_pos_mod(self):
        self.assertEqual(dt.ModDie(5, 3).multiply_str(2), '2D5+6')
    def test_moddie_multiply_str_neg_mod(self):
        self.assertEqual(dt.ModDie(5, -3).multiply_str(2), '2D5-6')

    def test_weighteddie_get_size(self):
        self.assertEqual(dt.WeightedDie({5:2, 2:5}).get_size(), 5)
    def test_weighteddie_get_weight(self):
        self.assertEqual(dt.WeightedDie({1:2, 3:5}).get_weight(), 7)
    def test_weighteddie_tuple_list(self):
        self.assertEqual(dt.WeightedDie({1:2, 3:4}).tuple_list(), [(1, 2), (3, 4)])
    def test_weighteddie_tuple_list_doesnt_return_zero_frequencies(self):
        self.assertEqual(dt.WeightedDie({1:2, 3:0}).tuple_list(), [(1, 2)])
    def test_weighteddie_weight_info(self):
        dic = dict((x, x+1) for x in range(1, 6, 2))
        weights_str = ('D5  W:12\n'+
                       '    a roll of 1 has a weight of 2\n' +
                       '    a roll of 2 has a weight of 0\n' +
                       '    a roll of 3 has a weight of 4\n' +
                       '    a roll of 4 has a weight of 0\n' +
                       '    a roll of 5 has a weight of 6')
        self.assertEqual(dt.WeightedDie(dic).weight_info(), weights_str)
    def test_weighteddie_str(self):
        self.assertEqual(str(dt.WeightedDie({1:1, 5:3})), 'D5  W:4')
    def test_weighteddie_repr(self):
        self.assertEqual(repr(dt.WeightedDie({1:1, 2:3})),
                         'WeightedDie({1: 1, 2: 3})')
    def test_weighteddie_repr_edge_case(self):
        die1 = dt.WeightedDie({1:1, 3:0})
        die2 = dt.WeightedDie({1:1, 2:0, 3:0})
        the_repr = 'WeightedDie({1: 1, 2: 0, 3: 0})'
        self.assertEqual(repr(die1), the_repr)
        self.assertEqual(repr(die2), the_repr)
        self.assertEqual(die1, die2)
    def test_weighteddie_multiply_str(self):
        self.assertEqual(dt.WeightedDie({1:1, 5:3}).multiply_str(2), '2D5  W:4')

    def test_modweighteddie_get_mod(self):
        self.assertEqual(dt.ModWeightedDie({1:2}, 3).get_modifier(), 3)
    def test_modweighteddie_tuple_list(self):
        self.assertEqual(dt.ModWeightedDie({1:2, 3:4}, -2).tuple_list(),
                         [(1-2, 2), (3-2, 4)])
    def test_modweighteddie_string_for_positive_mod(self):
        self.assertEqual(str(dt.ModWeightedDie({1:2}, 3)), 'D1+3  W:2')
    def test_modweighteddie_string_for_negative_mod(self):
        self.assertEqual(str(dt.ModWeightedDie({1:2}, -3)), 'D1-3  W:2')
    def test_modweighteddie_repr(self):
        self.assertEqual(repr(dt.ModWeightedDie({1:2}, -3)),
                         'ModWeightedDie({1: 2}, -3)')
    def test_modweighteddie_multiply_str_for_positive_mod(self):
        self.assertEqual(dt.ModWeightedDie({1:2}, 3).multiply_str(5), '5D1+15  W:2')
    def test_modweighteddie_multiply_str_for_negative_mod(self):
        self.assertEqual(dt.ModWeightedDie({1:2}, -3).multiply_str(5), '5D1-15  W:2')

    def test_strongdie_get_size(self):
        orig = dt.ModWeightedDie({1:2}, -3)
        self.assertEqual(dt.StrongDie(orig, 100).get_size(), orig.get_size())
    def test_strongdie_get_weight(self):
        orig = dt.ModWeightedDie({1:2}, -3)
        self.assertEqual(dt.StrongDie(orig, 100).get_weight(), orig.get_weight())
    def test_strongdie_get_multiplier(self):
        self.assertEqual(dt.StrongDie(dt.Die(3), 5).get_multiplier(), 5)
    def test_strongdie_get_original(self):
        self.assertEqual(dt.StrongDie(dt.Die(3), 5).get_original(), dt.Die(3))
    def test_strongdie_tuple_list(self):
        orig = dt.ModWeightedDie({1:2, 2:1}, 1)
        self.assertEqual(dt.StrongDie(orig, 100).tuple_list(),
                         [(200, 2), (300, 1)])
    def test_strongdie_weight_info(self):
        dic = dict((x, x+1) for x in range(1, 6, 2))
        weights_str = ('(D5  W:12)X10\n'+
                       '    a roll of 1 has a weight of 2\n' +
                       '    a roll of 2 has a weight of 0\n' +
                       '    a roll of 3 has a weight of 4\n' +
                       '    a roll of 4 has a weight of 0\n' +
                       '    a roll of 5 has a weight of 6')
        self.assertEqual(dt.StrongDie(dt.WeightedDie(dic), 10).weight_info(),
                         weights_str)
    def test_strongdie_multiply_str(self):
        orig = dt.ModWeightedDie({1:2}, -3)
        expected = '(5D1-15  W:2)X100'
        self.assertEqual(dt.StrongDie(orig, 100).multiply_str(5), expected)
    def test_strongdie_str(self):
        self.assertEqual(str(dt.StrongDie(dt.Die(7), 5)), '(D7)X5')
    def test_strongdie_repr(self):
        self.assertEqual(repr(dt.StrongDie(dt.Die(7), 5)), 'StrongDie(Die(7), 5)')
    def test_strongdie_expected_lt_cases_positive_tuple_list(self):
        die = dt.Die(5)
        self.assertEqual(die < dt.StrongDie(die, 2), True)
    def test_strongdie_expected_lt_cases_zero_tuple_list(self):
        die = dt.ModDie(5, -1)
        self.assertEqual(die < dt.StrongDie(die, 2), True)
    def test_strongdie_expected_lt_cases_negative_tuple_list(self):
        die = dt.ModDie(5, -3)
        self.assertEqual(die < dt.StrongDie(die, 2), False)

    def test_dicetable_inits_empty(self):
        table = dt.DiceTable()
        self.assertEqual(table.frequency_all(), [(0, 1)])
        self.assertEqual(table.get_list(), [])
    def test_dicetable_update_list_adds_same_die_to_die_already_in_table(self):
        table = dt.DiceTable()
        table.update_list(3, dt.Die(4))
        table.update_list(2, dt.Die(4))
        self.assertIn((dt.Die(4), 5), table.get_list())
    def test_dicetable_update_list_doesnt_combine_similar_dice(self):
        table = dt.DiceTable()
        table.update_list(3, dt.Die(3))
        table.update_list(2, dt.WeightedDie({1:1, 2:1, 3:1}))
        self.assertIn((dt.WeightedDie({1:1, 2:1, 3:1}), 2), table.get_list())
        self.assertIn((dt.Die(3), 3), table.get_list())
    def test_dicetable_updates_list_removes_die_with_zero_amount(self):
        table = dt.DiceTable()
        table.update_list(2, dt.Die(4))
        self.assertIn((dt.Die(4), 2), table.get_list())
        table.update_list(-2, dt.Die(4))
        self.assertEqual([], table.get_list())
    def test_dicetable_number_of_dice_reports_correctly(self):
        table = dt.DiceTable()
        table.update_list(5, dt.Die(4))
        self.assertEqual(table.number_of_dice(dt.Die(4)), 5)
    def test_dicetable_number_of_dice_reports_zero_when_dice_not_there(self):
        table = dt.DiceTable()
        self.assertEqual(table.number_of_dice(dt.Die(4)), 0)
    def test_weights_info_returns_empty_str_for_empty_table(self):
        table = dt.DiceTable()
        self.assertEqual(table.weights_info(), '')
    def test_weights_info_returns_appropriate_string(self):
        table = dt.DiceTable()
        table.update_list(2, dt.Die(4))
        table.update_list(5, dt.ModWeightedDie({1:10, 4:0}, 2))
        w_info = ('2D4\n    No weights\n\n5D4+10  W:10\n'+
                  '    a roll of 1 has a weight of 10\n'+
                  '    a roll of 2 has a weight of 0\n'+
                  '    a roll of 3 has a weight of 0\n'+
                  '    a roll of 4 has a weight of 0')
        self.assertEqual(table.weights_info(), w_info)
    def test_str_is_empty_for_empty_table(self):
        table = dt.DiceTable()
        self.assertEqual(str(table), '')
    def test_str_returns_appropriate_value(self):
        table = dt.DiceTable()
        table.update_list(2, dt.ModDie(4, -2))
        table.update_list(3, dt.Die(10))
        table.update_list(5, dt.ModWeightedDie({4:10}, 2))
        table_str = '2D4-4\n5D4+10  W:10\n3D10'
        self.assertEqual(str(table), table_str)

    def test_add_doesnt_add_zero_dice(self):
        table = dt.DiceTable()
        table.add_die(0, dt.Die(4))
        self.assertEqual(table.get_list(), [])
        self.assertEqual(table.frequency_all(), [(0, 1)])
    def test_add_adds_correct_dice(self):
        table = dt.DiceTable()
        table.add_die(2, dt.Die(4))
        self.assertEqual(table.get_list(), [(dt.Die(4), 2)])
        freq_all = [(2, 1), (3, 2), (4, 3), (5, 4), (6, 3), (7, 2), (8, 1)]
        self.assertEqual(table.frequency_all(), freq_all)
    def test_remove_die_removes_correct_dice(self):
        table = dt.DiceTable()
        table.add_die(5, dt.Die(4))
        table.remove_die(3, dt.Die(4))
        self.assertEqual(table.get_list(), [(dt.Die(4), 2)])
        freq_all = [(2, 1), (3, 2), (4, 3), (5, 4), (6, 3), (7, 2), (8, 1)]
        self.assertEqual(table.frequency_all(), freq_all)
    def test_remove_die_can_remove_all_the_dice(self):
        table = dt.DiceTable()
        table.add_die(2, dt.Die(4))
        table.remove_die(2, dt.Die(4))
        self.assertEqual(table.get_list(), [])
        self.assertEqual(table.frequency_all(), [(0, 1)])
    def test_remove_die_raises_error_if_die_not_in_table(self):
        table = dt.DiceTable()
        with self.assertRaises(ValueError) as cm:
            table.remove_die(1, dt.Die(4))
        self.assertEqual(cm.exception.args[0],
                         'dice not in table, or removed too many dice')
    def test_remove_die_raises_error_if_too_many_dice_removed(self):
        table = dt.DiceTable()
        table.add_die(3, dt.Die(4))
        with self.assertRaises(ValueError) as cm:
            table.remove_die(4, dt.Die(4))
        self.assertEqual(cm.exception.args[0],
                         'dice not in table, or removed too many dice')

if __name__ == '__main__':
    unittest.main()




# pylint: disable=missing-docstring, invalid-name, too-many-public-methods
'''tests for the longintmath.py module'''
from __future__ import absolute_import

import unittest
import dicetables.longintmath as lim


FLOAT_BIG = 1e+300
FLOAT_SMALL = 1e+100
LONG_BIG = 10**1000
class TestLongIntMathFunctionsDivTimesPow(unittest.TestCase):

    def test_long_int_div_returns_zero_when_answer_power_below_neg_300ish(self):
        self.assertEqual(lim.long_int_div(FLOAT_BIG, LONG_BIG), 0)

    def test_long_int_div_long_long_makes_float(self):
        result = lim.long_int_div(10**300 * LONG_BIG, LONG_BIG)
        self.assertAlmostEqual(result, 10**300, delta=10**290)
        self.assertIsInstance(result, float)
    def test_long_int_div_long_long_makes_float_with_negative_num(self):
        result = lim.long_int_div(-10**300 * LONG_BIG, LONG_BIG)
        self.assertAlmostEqual(result, -10**300, delta=10**290)
        self.assertIsInstance(result, float)
    def test_long_int_div_float_float_makes_long(self):
        result = lim.long_int_div(FLOAT_BIG, 1 / FLOAT_SMALL)
        self.assertAlmostEqual(result, 10**400, delta=10**390)
    def test_long_int_div_float_float_makes_long_with_negative_num(self):
        result = lim.long_int_div(FLOAT_BIG, -1 / FLOAT_SMALL)
        self.assertAlmostEqual(result, -10**400, delta=10**390)
    def test_long_int_div_long_long_makes_negative_power_float(self):
        result = lim.long_int_div(LONG_BIG, LONG_BIG * 10**200)
        self.assertAlmostEqual(result, 10**-200, delta=10**-210)

    def test_long_int_times_makes_large_value(self):
        result = lim.long_int_times(LONG_BIG, FLOAT_BIG)
        self.assertAlmostEqual(result, 10**1300, delta=10**1290)
    def test_long_int_times_makes_between_one_and_zero(self):
        result = lim.long_int_times(FLOAT_SMALL, 1 / FLOAT_BIG)
        self.assertAlmostEqual(result, 10**-200, delta=10**-210)
    def test_long_int_times_makes_negative_number(self):
        result = lim.long_int_times(FLOAT_SMALL, -FLOAT_BIG)
        self.assertAlmostEqual(result, -10**400, delta=10**390)

    def test_long_int_pow_makes_one(self):
        result = lim.long_int_pow(LONG_BIG, 0)
        self.assertEqual(result, 1)
    def test_long_int_pow_makes_zero(self):
        result = lim.long_int_pow(LONG_BIG, -1)
        self.assertEqual(result, 0)
    def test_long_pow_makes_between_one_and_zero(self):
        result = lim.long_int_pow(FLOAT_SMALL, -2)
        self.assertAlmostEqual(result, 1e-200, delta=10**-210)
    def test_long_pow_with_float_power(self):
        result = lim.long_int_pow(LONG_BIG, 2.5)
        self.assertAlmostEqual(result, 10**2500, delta=10**2490)

class TestLongIntTable(unittest.TestCase):
    def setUp(self):
        self.identity_a = lim.LongIntTable({0:1})
        self.identity_b = lim.LongIntTable({0:1})

    def tearDown(self):
        del self.identity_a
        del self.identity_b

    def test_values_sorts_and_removes_zeros(self):
        test_table = lim.LongIntTable({0:1, 1:0})
        self.assertEqual(test_table.values(), [0])
    def test_values_returns_empty_list_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(empty.values(), [])

    def test_values_min_returns_min_value(self):
        min_val_neg_one = lim.LongIntTable({1:1, -1:1})
        self.assertEqual(min_val_neg_one.values_min(), -1)
    def test_values_max_returns_max_value(self):
        max_val_neg_one = lim.LongIntTable({-1:1, -2:1})
        self.assertEqual(max_val_neg_one.values_max(), -1)
    def test_values_range_returns_minmax(self):
        zero_to_two = lim.LongIntTable({0:2, 1:1, 2:5, 4:0})
        self.assertEqual(zero_to_two.values_range(), (0, 2))

    def test_values_min_returns_none_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertIsNone(empty.values_min())
    def test_values_max_returns_none_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertIsNone(empty.values_max())
    def test_values_range_returns_none_none_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(empty.values_range(), (None, None))


    def test_frequency_returns_value_frequency_as_tuple(self):
        zero_three = lim.LongIntTable({0:3})
        self.assertEqual(zero_three.frequency(0), (0, 3))
    def test_frequency_returns_zero_frequency_for_empty_value(self):
        zero_three = lim.LongIntTable({0:3})
        self.assertEqual(zero_three.frequency(100), (100, 0))

    def test_frequency_range_returns_correct_tuple_list(self):
        table = lim.LongIntTable({1:1, 2:2})
        self.assertEqual(table.frequency_range(0, 4),
                         [(0, 0), (1, 1), (2, 2), (3, 0)])

    def test_frequency_all_returns_for_normal_case(self):
        table = lim.LongIntTable({1:1, 2:2})
        self.assertEqual(table.frequency_all(),
                         [(1, 1), (2, 2)])
    def test_frequency_all_return_empty_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(empty.frequency_all(), [])
    def test_frequency_all_sorts_and_does_not_return_zero_frequencies(self):
        table = lim.LongIntTable({1:2, 2:0, -1:2})
        self.assertEqual(table.frequency_all(), [(-1, 2), (1, 2)])

    def test_frequency_highest_returns_either_tuple_with_highest(self):
        table = lim.LongIntTable({-1:5, 0:1, 2:5})
        self.assertIn(table.frequency_highest(), [(-1, 5), (2, 5)])
    def test_frequency_highest_returns_only_the_highest(self):
        table = lim.LongIntTable({-1:5, 0:1, 2:3})
        self.assertEqual(table.frequency_highest(), (-1, 5))
    def test_frequency_highest_handles_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(empty.frequency_highest(), (None, 0))

    def test_total_frequency_is_zero_for_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(empty.total_frequency(), 0)
    def test_total_frequency_returns_correct_value_table(self):
        table = lim.LongIntTable({1:2, 3:4})
        self.assertEqual(table.total_frequency(), 2+4)


    def test_string_returns_min_to_max(self):
        table = lim.LongIntTable({-1:1, 2:1, 5:1})
        self.assertEqual(str(table), 'table from -1 to 5')
    def test_string_is_in_order_and_ignores_high_zero_values(self):
        table = lim.LongIntTable({2:0, 1:1, -1:1, -2:0})
        self.assertEqual(str(table), 'table from -1 to 1')
    def test_string_of_empty_table(self):
        empty = lim.LongIntTable({})
        self.assertEqual(str(empty), 'table from None to None')


    def test_mean_normal_case(self):
        table = lim.LongIntTable({-1:5, 1:5})
        self.assertEqual(table.mean(), 0)
    def test_mean_empty_list_raises_zero_division_error(self):
        empty = lim.LongIntTable({})
        with self.assertRaises(ZeroDivisionError) as cm:
            empty.mean()
        self.assertEqual(cm.exception.args[0], 'there are no values in the table')
    def test_mean_with_non_uniform_table(self):
        table = lim.LongIntTable({1:2, 2:5})
        mean = (2 + 10) / float(2 + 5)
        self.assertEqual(table.mean(), mean)
    def test_mean_with_large_number_table(self):
        table = lim.LongIntTable({1:2*10**1000, 2:2*10**1000})
        self.assertEqual(table.mean(), 1.5)


    def test_stddev_for_table_with_highest_frequency_below_cutoff(self):
        low_freq = lim.LongIntTable({2:1, -2:1, 1:1, -1:1})
        self.assertEqual(low_freq.stddev(), round((10/4.)**0.5, 4))
    def test_stddev_for_below_cutoff_table_with_more_decimals(self):
        low_freq = lim.LongIntTable({2:1, -2:1, 1:1, -1:1})
        self.assertEqual(low_freq.stddev(decimal_place=10), round((10/4.)**0.5, 10))

    def test_stddev_for_table_with_highest_frequency_above_cutoff(self):
        high_freq = lim.LongIntTable({2:10**50, -2:10**50, 1:10**50, -1:10**50})
        self.assertEqual(high_freq.stddev(), round((10/4.)**0.5, 4))
    def test_stddev_for_above_cutoff_table_with_more_decimals(self):
        high_freq = lim.LongIntTable({2:10**50, -2:10**50, 1:10**50, -1:10**50})
        self.assertEqual(high_freq.stddev(decimal_place=10), round(2.5**0.5, 10))


    def test_merge_adds_old_vals_and_makes_new_vals(self):
        table = lim.LongIntTable({1:1, 2:1, 3:1})
        table.merge({-1:1, 0:1, 1:1, 2:1}.items())
        self.assertEqual(table.frequency_all(),
                         [(-1, 1), (0, 1), (1, 2), (2, 2), (3, 1)])

    def test_update_frequency(self):
        table = lim.LongIntTable({1:1, 2:2})
        table.update_frequency(2, 5)
        self.assertEqual(table.frequency(2)[1], 5)
    def test_update_frequency_when_update_value_not_in_table(self):
        table = lim.LongIntTable({1:1})
        table.update_frequency(2, 5)
        self.assertEqual(table.frequency(2)[1], 5)
    def test_update_value_ow(self):
        table = lim.LongIntTable({1:100, 2:2})
        table.update_value_ow(1, 2)
        self.assertEqual(table.frequency_all(), [(2, 100)])
    def test_update_value_add(self):
        table = lim.LongIntTable({1:100, 2:2})
        table.update_value_add(1, 2)
        self.assertEqual(table.frequency_all(), [(2, 102)])

    def test_add_empty_list_error(self):
        identity = lim.LongIntTable({0:1})
        with self.assertRaises(ValueError) as cm:
            identity.add(1, [(1, 0)])
        self.assertEqual(cm.exception.args[0], 'cannot add an empty list')
    def test_add_negative_times_error(self):
        identity = lim.LongIntTable({0:1})
        with self.assertRaises(ValueError) as cm:
            identity.add(-1, [(1, 1)])
        self.assertEqual(cm.exception.args[0], 'times must be a positive int')
    def test_add_negative_frequencies_error(self):
        identity = lim.LongIntTable({0:1})
        with self.assertRaises(ValueError) as cm:
            identity.add(1, [(1, -1)])
        self.assertEqual(cm.exception.args[0], 'frequencies may not be negative')
    def test_add_errors_do_not_mutate_table(self):
        identity = lim.LongIntTable({0:1})
        try:
            identity.add(1, [(1, 0)])
        except ValueError:
            pass
        try:
            identity.add(-1, [(1, 1)])
        except ValueError:
            pass
        try:
            identity.add(1, [(1, -1)])
        except ValueError:
            pass
        self.assertEqual(identity.frequency_all(), [(0, 1)])
#the next two tests test that add works for the two cases for _fastest()
#_fastest() is found in the the add() method
#for details of adding ints vs tuples, see testing_add_speed.py
    def test_add_works_with_low_total_frequency_to_number_of_vals(self):
        low_ratio_tuples = [(1, 1), (2, 1)]
        self.identity_a.add(1, low_ratio_tuples)
        self.assertEqual(self.identity_a.frequency_all(), low_ratio_tuples)
    def test_add_works_with_high_total_frequency_to_number_of_vals(self):
        high_ratio_tuples = [(1, 10**1000), (2, 10**1000)]
        self.identity_a.add(1, high_ratio_tuples)
        self.assertEqual(self.identity_a.frequency_all(), high_ratio_tuples)

    def test_one_multiple_add_is_multiple_single_adds(self):
        tuples = [(1, 2), (3, 4)]
        self.identity_a.add(1, tuples)
        self.identity_a.add(1, tuples)
        self.identity_b.add(2, tuples)
        self.assertEqual(self.identity_a.frequency_all(),
                         self.identity_b.frequency_all())
    def test_add_adds_correctly(self):
        tuple_list = [(1, 2), (2, 2)]
        self.identity_a.add(2, tuple_list)
        self.assertEqual(self.identity_a.frequency_all(),
                         [(2, 4), (3, 8), (4, 4)])
    def test_add_works_with_long_large_number_list(self):
        tuple_list = [(x, 10**1000) for x in range(-1000, 1000)]
        self.identity_a.add(1, tuple_list)
        self.assertEqual(self.identity_a.frequency_all(), tuple_list)
    def test_remove_removes_correctly(self):
        arbitrary_tuples = [(-5, 2), (0, 5), (3, 10)]
        self.identity_a.add(5, arbitrary_tuples)
        self.identity_b.add(10, arbitrary_tuples)
        self.identity_b.remove(5, arbitrary_tuples)
        self.assertEqual(self.identity_a.frequency_all(),
                         self.identity_b.frequency_all())
    def test_remove_works_for_large_numbers(self):
        arbitrary_tuples_large = [(-5, 10**500), (0, 5*10**700), (3, 2**1000)]

        self.identity_a.add(1, arbitrary_tuples_large)
        self.identity_b.add(2, arbitrary_tuples_large)
        self.identity_b.remove(1, arbitrary_tuples_large)
        self.assertEqual(self.identity_a.frequency_all(),
                         self.identity_b.frequency_all())

    def test_add_adds_same_regardless_of_order(self):
        arbitrary_a = [(1, 2), (3, 10**456)]
        arbitrary_b = [(0, 5), (-1, 2)]
        self.identity_a.add(1, arbitrary_a)
        self.identity_a.add(2, arbitrary_b)
        self.identity_b.add(2, arbitrary_b)
        self.identity_b.add(1, arbitrary_a)
        self.assertEqual(self.identity_a.frequency_all(),
                         self.identity_b.frequency_all())

    def test_remove_removes_same_regardless_of_order(self):
        arbitrary_a = [(-1, 2), (3, 5)]
        arbitrary_b = [(100, 4), (0, 9)]
        self.identity_a.add(5, arbitrary_a)
        self.identity_a.add(5, arbitrary_b)
        self.identity_b.add(1, arbitrary_b)
        self.identity_b.add(1, arbitrary_a)
        self.identity_a.remove(3, arbitrary_a)
        self.identity_a.remove(3, arbitrary_b)
        self.identity_a.remove(1, arbitrary_b)
        self.identity_a.remove(1, arbitrary_a)
        self.assertEqual(self.identity_a.frequency_all(),
                         self.identity_b.frequency_all())


if __name__ == '__main__':
    unittest.main()




'''a module solely for finding how add_a_list and add_tuple_list compare.
it's effectively the empirical proof for how LongIntTable.add() chooses
the fastest method with it's _fastest() function.'''
import dicetables.longintmath as lim
import time
import matplotlib.pyplot as plt
import numpy as np
import random

def gen_one_point(val_list, loc='mid'):
    '''a generator for a list of tuples. starts list of tuples (val, 1) for each
    val in list. add one to loc = "start", "mid" or "end"'''
    lst = [(val, 1) for val in val_list]
    if loc == 'start':
        index = 0
    elif loc == 'end':
        index = len(lst) - 1
    else:
        index = len(val_list) // 2
    index_val = lst[index][0]
    while True:
        yield lst
        current_freq = lst[index][1]
        lst[index] = (index_val, current_freq + 1)

def gen_n_points(val_list, num_points):
    '''input is a list and the number of points getting added.  start with tuple
    list of (val, 1) for val in list.  adds one to the frequency of number of
    pts.'''
    lst = [(val, 1) for val in val_list]
    factor = len(val_list) / float(num_points + 1)
    indexes = [int(factor * multiplier) for multiplier in range(1, num_points + 1)]
    while True:
        yield lst
        for index in indexes:
            val, freq = lst[index]
            lst[index] = (val, freq + 1)

def gen_random_point(val_list):
    '''input is a list of vals.  generates list of (val, 1). then each iteration
    add one to a random freq'''
    lst = [(val, 1) for val in val_list]
    length = len(lst)
    while True:
        yield lst
        index = random.randrange(0, length)
        val, freq = lst[index]
        lst[index] = (val, freq + 1)

def add_int_lists(table, num_times, lst):
    '''repeat the add_a_list function num_times times.'''
    for _ in range(num_times):
        table._add_a_list(lst)

def add_tuple_lists(table, num_times, lst):
    '''num_times is an int.  repeat add_tuple_list that many times.'''
    for _ in range(num_times):
        table._add_tuple_list(lst)

def one_time_trial(tuple_list, num_adds):
    '''add t_list to identity LongIntTable num_adds times, using _add_a_list and
    _add_tuple_list.  returns the ratio of sum_of_freq/num_of_values, and the
    time for each funtion to do the adding.'''
    tuple_list.sort()
    lst = []
    for val, freq in tuple_list:
        lst = lst + [val] * freq

    freq_val_ratio = len(lst)/float(len(tuple_list))

    id_table_a = lim.LongIntTable({0:1})
    start_a = time.clock()
    add_int_lists(id_table_a, num_adds, lst)
    list_time = time.clock() - start_a

    id_table_b = lim.LongIntTable({0:1})
    start_b = time.clock()
    add_tuple_lists(id_table_b, num_adds, tuple_list)
    tuple_time = time.clock() - start_b

    return freq_val_ratio, tuple_time, list_time

def time_trial(generator, adds_per_trial):
    '''run one time trial, then run again while advancing the generator. test
    runs about 1.8 times length from start to equal times. outputs x-axis and
    two y-axes.'''
    ratios = []
    tuple_times = []
    list_times = []
    count = 1
    print 'please wait for the count-up/down to reach zero'
    while count > 0:
        ratio, tup_time, lst_time = one_time_trial(generator.next(),
                                                   adds_per_trial)
        print count
        if tup_time > lst_time:
            count += 1
        else:
            count -= 1.2
        ratios.append(ratio)
        tuple_times.append(tup_time)
        list_times.append(lst_time)
    return ratios, tuple_times, list_times

def plot_trial(ratios, tuples, lists, title='none', figure=1):
    '''plot x-axis = ratios.  y-axis = tuples and y-axis = lists. fit curves to
    ax + b and return the intesection of two curves'''
    plt.figure(figure)
    plt.plot(ratios, tuples, 'bo-', label='tuple add')
    plt.plot(ratios, lists, 'r*-', label='list add')
    plt.ylabel('time')
    plt.xlabel('num_freq over num_vals')

    plt.legend()
    intersection, tup_fit, lst_fit = polyfit_and_intersection(ratios,
                                                              tuples, lists)
    title = title + '\nintersection = %s' % (intersection)
    plt.title(title)
    plt.plot(ratios, tup_fit, 'c-')
    plt.plot(ratios, lst_fit, 'c-')
    return intersection

def polyfit_and_intersection(ratios, tuples, lists):
    '''fits tuples and list linearly to ratios. returns the intersection of two
    fits and two lists of y values to plot with ratios'''
    tup_slope, tup_const = np.polyfit(ratios, tuples, 1)
    lst_slope, lst_const = np.polyfit(ratios, lists, 1)
    intersection = (tup_const - lst_const) / (lst_slope - tup_slope)
    tuple_polyfit = [(tup_slope * x + tup_const) for x in ratios]
    lst_polyfit = [(lst_slope * x + lst_const) for x in ratios]
    return intersection, tuple_polyfit, lst_polyfit

def random_list(num_vals):
    '''num_vals is how many elements in your list.  generates list start at
    -10 to 10 with a step of 1 - 5'''
    start = random.randrange(-10, 11)
    step = random.randrange(1, 6)
    return range(start, step*num_vals + start, step)

def random_generator(num_vals, n_points_only=False):
    '''outputs a random generator of len(lst) = num_vals'''
    start_list = random_list(num_vals)
    if n_points_only:
        gen_str = 'n_points'
    else:
        gen_str = random.choice(['one_point', 'random_point', 'n_points'])
    if gen_str == 'one_point':
        loc = random.choice(['start', 'mid', 'end'])
        generator = gen_one_point(start_list, loc)
        out_str = '%s\n%s at %s' % (start_list, gen_str, loc)
    elif gen_str == 'random_point':
        generator = gen_random_point(start_list)
        out_str = '%s\n%s' % (start_list, gen_str)
    else:
        min_pts = len(start_list) // 10
        max_pts = len(start_list) // 5
        if min_pts < 1:
            min_pts = 1
        if max_pts < 2:
            max_pts = 2
        num_points = random.randrange(min_pts, max_pts + 1)
        generator = gen_n_points(start_list, num_points)
        out_str = '%s\n%s n=%s' % (start_list, gen_str, num_points)
    return generator, out_str

def random_trial(num_vals, adds_per_trial, figure=1, n_points_only=False):
    '''make a random generator of a random list with num_vals elements. add that
    list num_adds times per trial and plot results and return intesection'''
    generator, title = random_generator(num_vals, n_points_only)
    title = title + '  adds=%s  vals=%s' % (adds_per_trial, num_vals)
    ratio, tuples, lists = time_trial(generator, adds_per_trial)
    return plot_trial(ratio, tuples, lists, title, figure)

from os import getcwd

def get_welcome():
    '''return welcom_message.txt'''
    try:
        welcome_file_name = getcwd() + '\\' + 'welcome_message.txt'
        welcome_file = open(welcome_file_name, 'r')
        welcome_message = welcome_file.read()
    except IOError:
        welcome_message = 'took a guess where "welcome_' \
                          'message.txt" was, and I was wrong.'
    return welcome_message
def get_int(question):
    '''makes sure user input is an int. quit if "q"'''
    while True:
        answer = raw_input(question + '>>>')
        if answer == 'q':
            raise SystemExit
        try:
            output = int(answer)
            return output
        except ValueError:
            print 'must be int OR "q" to quit'
            continue

def quick_and_dirty_ui():
    '''a UI to demonstrate add speeds'''
    print get_welcome()
    this_will_take_forever = 2500
    too_short_for_accuracy = 1200
    needs_many_points = 50
    max_vals = 300
    min_vals = 3
    figure = 0

    while True:
        figure += 1
        plt.ion()
        num_vals_question = (('how many values in your list?'+
                              '\nplease input int between %s and %s\n') %
                             (min_vals, max_vals))
        num_vals = get_int(num_vals_question)
        if num_vals > max_vals:
            print 'num_vals too big. now = %s' % (max_vals)
            num_vals = max_vals
        if num_vals < min_vals:
            print 'num_vals too small. now = %s' % (min_vals)
            num_vals = min_vals

        max_adds = this_will_take_forever // num_vals
        min_adds = 1 + too_short_for_accuracy // num_vals
        num_adds_question = ('please input the number of adds per trial\n' +
                             'an int between %s and %s\n' % (min_adds, max_adds))
        num_adds = get_int(num_adds_question)
        if num_adds < min_adds:
            print 'num_adds too small. now = %s' % (min_adds)
            num_adds = min_adds
        if num_adds > max_adds:
            print 'num_adds too big. now = %s' % (max_adds)
            num_adds = max_adds

        if num_vals >= needs_many_points:
            n_points_only = True
        else:
            n_points_only = False
        intersection = random_trial(num_vals, num_adds, figure, n_points_only)
        print 'the graphs intersect at %s' % (intersection)
        plt.pause(0.1)

def tst_num_adds(num_vals, start_add, stop_add):
    '''shows how the intersecion varies with different numbers of adds'''
    start_list = random_list(num_vals)
    num_points = num_vals // 5
    if num_points == 0:
        num_points = 1
    out_lst = []
    x_axis = range(start_add, stop_add + 1)
    for adds in x_axis:
        print 'countdown %s' % (x_axis[-1] - adds)
        generator = gen_n_points(start_list, num_points)
        ratios, tuples, lists = time_trial(generator, adds)
        ans = polyfit_and_intersection(ratios, tuples, lists)
        out_lst.append(ans[0])
    plt.figure(1)
    plt.plot(x_axis, out_lst, 'bo-', label=str(start_list))
    plt.xlabel('%s to %s' % (start_add, stop_add))
    plt.ylabel('intersections')
    return out_lst

def tst_num_vals(star_vals, stop_vals, num_adds):
    '''shows how the intesection varies with different number of values in a
    list'''
    x_axis = range(star_vals, stop_vals + 1)
    out_lst = []
    for num_vals in x_axis:
        print 'countdown %s' % (x_axis[-1] - num_vals)
        num_points = num_vals // 5
        start_list = random_list(num_vals)
        if num_points == 0:
            num_points = 1
        generator = gen_n_points(start_list, num_points)
        ratios, tuples, lists = time_trial(generator, num_adds)
        ans = polyfit_and_intersection(ratios, tuples, lists)
        out_lst.append(ans[0])
        plt.figure(2)
    plt.plot(x_axis, out_lst, 'bo-', label=str(start_list))
    plt.xlabel('%s to %s' % (star_vals, stop_vals))
    plt.ylabel('intersections')
    return out_lst

if __name__ == '__main__':
    quick_and_dirty_ui()



from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='dicetables',
      version='0.4.3',
      description='dice statistics module',
      long_description=readme(),
      keywords='dice, die, statistics, table, probability, combinations',
      url='http://github.com/eric-s-s/dice-tables',
      author='Eric Shaw',
      author_email='shaweric01@gmail.com',
      license='MIT',
      classifiers=[
        'Development Status :: 4 - Beta',
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Games/Entertainment :: Role-Playing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
	'Programming Language :: Python :: 3',
      ],
      packages=['dicetables', 'tests'],
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      zip_safe=False)
